//
//  CASCBridge.m
//  CleverAdsSolutions Construct
//
//  Copyright © 2020 Clever Ads Solutions. All rights reserved.
//

#import "CASCBridge.h"
#import <Cordova/CDVViewController.h>

// ----- Constants
static NSInteger const BANNER = 0;
static NSInteger const INTERSTITIAL = 1;
static NSInteger const REWARDED = 2;
static NSInteger const APP_RETURN = 3;

@implementation CASCBridge
{
    NSInteger bannerPositionId;
}

- (void) initialize:(CDVInvokedUrlCommand *)command {
    NSDictionary *mode = [command argumentAtIndex:0];

    NSString *managerID = [mode valueForKey:@"managerId"];
    BOOL isTestMode = [[mode valueForKey:@"testMode"] boolValue];
    int enableAdTypes = [[mode valueForKey:@"enableAdTypes"] intValue];
    BOOL isAppReturn = [[mode valueForKey:@"returnAds"] boolValue];
    NSString *platform = [mode valueForKey:@"frameworkName"];
    NSString *version = [mode valueForKey:@"frameworkVersion"];

    [self initializeMethod:managerID isTestMode:isTestMode enableAdTypes:enableAdTypes isAppReturn: isAppReturn platform:platform version:version command:command];
}

- (void) initializeWithDefaultParameters:(CDVInvokedUrlCommand *)command {
    NSDictionary *mode = [command argumentAtIndex:0];

    NSString *managerID = [mode valueForKey:@"managerId"];
    BOOL isTestMode = [[mode valueForKey:@"testMode"] boolValue];
    int enableAdTypes = [[mode valueForKey:@"enableAdTypes"] intValue];
    BOOL isAppReturn = [[mode valueForKey:@"returnAds"] boolValue];
    BOOL isDebugMode = [[mode valueForKey:@"debugMode"] boolValue];
    BOOL isAllowInterstitialAdsWhenRewardedCostAreLower = [[mode valueForKey:@"isAllowInterstitialAdsWhenRewardedCostAreLower"] boolValue];
    BOOL isValidateIntegration = [[mode valueForKey:@"validateIntegration"] boolValue];
    NSInteger GDPR = [[mode valueForKey:@"GDPR"] intValue];
    NSInteger CCPA = [[mode valueForKey:@"CCPA"] intValue];
    NSInteger taggedAudience = [[mode valueForKey:@"taggedAudience"] intValue];
    NSInteger loadingMode = [[mode valueForKey:@"loadingMode"] intValue];
    NSInteger interstitialInterval = [[mode valueForKey:@"interstitialInterval"] intValue];
    NSInteger refreshBannerDelay = [[mode valueForKey:@"refreshBannerDelay"] intValue];
    BOOL analyticsCollectionEnable = [[mode valueForKey:@"analyticsCollectionEnable"] boolValue];

    NSString *platform = [mode valueForKey:@"frameworkName"];
    NSString *version = [mode valueForKey:@"frameworkVersion"];

    [self initializeMethod:managerID isTestMode:isTestMode enableAdTypes:enableAdTypes isAppReturn: isAppReturn platform:platform version:version command:command];

    if (isValidateIntegration)
        [CAS validateIntegration];

    [[CAS settings] setDebugMode:isDebugMode];
    [[CAS settings] setInterstitialAdsWhenVideoCostAreLowerWithAllow:isAllowInterstitialAdsWhenRewardedCostAreLower];
    [[CAS settings] updateUserWithConsent: (CASConsentStatus)GDPR];
    [[CAS settings] updateCCPAWithStatus: (CASCCPAStatus)CCPA];
    [[CAS settings] setTaggedWithAudience:(CASAudience)taggedAudience];
    [[CAS settings] setLoadingWithMode:(CASLoadingManagerMode)loadingMode];
    [[CAS settings] setInterstitialWithInterval:interstitialInterval];
    [[CAS settings] setBannerRefreshWithInterval:refreshBannerDelay];
    [[CAS settings] setAnalyticsCollectionWithEnabled:analyticsCollectionEnable];

}

- (void) initializeMethod: (NSString *)managerId
               isTestMode:(BOOL)isTestMode
            enableAdTypes:(int)enableAdTypes
              isAppReturn:(BOOL)isAppReturn
                 platform:(NSString *)platform
                  version:(NSString *)version
                  command: (CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    self.mediationManager = [CAS createWithManagerID:managerId
                                         enableTypes:enableAdTypes
                                          demoAdMode:isTestMode
                                              onInit:^(BOOL succses, NSString *_Nullable error) {
        if (succses) {
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        }
        else {
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error];
        }

        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }];

    bannerPositionId = 3;
    [[CAS settings] setPluginPlatformWithName:platform version:version];

    _bannerCallback = [[CASCBannerCallback alloc] initWithCommandDelegate:self.commandDelegate withViewController:self.viewController];
    _interstitialCallback = [[CASCCallback alloc] initWithCommandDelegate:self.commandDelegate withCASType:INTERSTITIAL withViewController:self.viewController];
    _rewardedCallback = [[CASCCallback alloc] initWithCommandDelegate:self.commandDelegate withCASType:REWARDED withViewController:self.viewController];

    if (isAppReturn) {
        _appReturnCallback = [[CASCCallback alloc] initWithCommandDelegate:self.commandDelegate withCASType:APP_RETURN withViewController:self.viewController];

        [_mediationManager enableAppReturnAdsWith:_appReturnCallback];
    }

    [_mediationManager setAdLoadDelegate:self];
}

- (void) setDebugMode:(CDVInvokedUrlCommand *)command {
    [[CAS settings] setDebugMode:[command argumentAtIndex:0]];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) skipNextReturnAds:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        [_mediationManager skipNextAppReturnAds];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) loadAd:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSNumber *adsType = [command argumentAtIndex:0];
        NSInteger type = [adsType intValue];

        if (type == BANNER) {
            if (!self.bannerView) {
                [self createBannerView];
                [self.bannerView setHidden:YES];
            }
            [self.bannerView loadNextAd];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else if (type == INTERSTITIAL) {
            [_mediationManager loadInterstitial];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else if (type == REWARDED) {
            [_mediationManager loadRewardedAd];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else {
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:[@"Cordova bridge call load ad for unknow type id: " stringByAppendingString: [adsType stringValue]]];
        }
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) showAd:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSNumber *adsType = [command argumentAtIndex:0];
        NSInteger type = [adsType intValue];

        if (type == BANNER) {
            if (self.bannerView) {
                [self.bannerView setHidden:NO];
            } else {
                [self createBannerView];
            }
            [self setBannerPosition:bannerPositionId];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else if (type == INTERSTITIAL) {
            [_mediationManager presentInterstitialFromRootViewController:self.viewController callback:_interstitialCallback];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else if (type == REWARDED) {
            [_mediationManager presentRewardedAdFromRootViewController:self.viewController callback:_rewardedCallback];
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
        } else {
            result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:[@"Cordova bridge call load ad for unknow type id: " stringByAppendingString: [adsType stringValue]]];
        }
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) hideBanner:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        [_bannerView setHidden:YES];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) isAdReady:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
        [data setObject:[NSNumber numberWithBool:[_mediationManager isAdReadyWithType:(CASType)[command argumentAtIndex:0]]] forKey:@"isReady"];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) getLastActiveMediation:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
        [data setObject:[NSNumber numberWithBool:[_mediationManager getLastActiveMediationWithType:(CASType)[command argumentAtIndex:0]]] forKey:@"lastActiveMediation"];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    } @catch(id anException) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anException];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) isEnabled:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
        [data setObject:[NSNumber numberWithBool:[_mediationManager isEnabledWithType:(CASType)[command argumentAtIndex:0]]] forKey:@"isEnable"];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    } @catch(id anException) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anException];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) enableAd:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSNumber *type = [command argumentAtIndex:0];
        BOOL enable = [command argumentAtIndex:1];

        [_mediationManager setEnabled:enable type:(CASType)type];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:anExcpeption];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) setBannerSizeId:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSNumber *bannerSize = [command argumentAtIndex:0];
        [self setBannerSize:[bannerSize intValue]];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsArray:command.arguments];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) setBannerPositionId:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;

    @try {
        NSNumber *bannerPosition = [command argumentAtIndex:0];
        [self setBannerPosition:[bannerPosition intValue]];

        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    } @catch(id anExcpeption) {
        result = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsArray:command.arguments];
    } @finally {
        [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
    }
}

- (void) validateIntegration:(CDVInvokedUrlCommand *)command {
    [CAS validateIntegration];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setLastPageAdContent:(CDVInvokedUrlCommand *)command {
    NSString *json = [command argumentAtIndex:0];

    if (json == nil || json.length == 0) {
        [_mediationManager setLastPageAdContent:nil];
    }

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setBannerInSafeArea:(CDVInvokedUrlCommand *)command {
    __block CDVPluginResult *result = nil;
    result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];

    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

#pragma mark - Clever Ads Solutions Settings

- (void) getSDKVersion:(CDVInvokedUrlCommand *)command {
    NSString *sdkVersion = [CAS getSDKVersion];

    NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
    [data setObject:sdkVersion forKey:@"isEnable"];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setUserConsentStatus:(CDVInvokedUrlCommand *)command  {
    NSNumber *type = [command argumentAtIndex:0];
    [[CAS settings] updateUserWithConsent:(CASConsentStatus)type];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setDoNotSellStatus:(CDVInvokedUrlCommand *)command {
    NSNumber *type = [command argumentAtIndex:0];
    [[CAS settings] updateCCPAWithStatus:(CASCCPAStatus)type];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setTaggedAudience:(CDVInvokedUrlCommand *)command {
    NSNumber *type = [command argumentAtIndex:0];
    [[CAS settings] setTaggedWithAudience:(CASAudience)type];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setMutedAdSounds:(CDVInvokedUrlCommand *)command {
    BOOL enable = [command argumentAtIndex:0];
    [[CAS settings] setMuteAdSoundsTo:enable];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setLoadingMode:(CDVInvokedUrlCommand *)command {
    NSNumber *type = [command argumentAtIndex:0];
    [[CAS settings] setLoadingWithMode:(CASLoadingManagerMode)type];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setRefreshBannerDelay:(CDVInvokedUrlCommand *)command {
    NSNumber *delay = [command argumentAtIndex:0];
    [[CAS settings] setBannerRefreshWithInterval:[delay intValue]];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setInterstitialInterval:(CDVInvokedUrlCommand *)command {
    NSNumber *interval = [command argumentAtIndex:0];
    [[CAS settings] setBannerRefreshWithInterval:[interval intValue]];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) restartInterstitialInterval:(CDVInvokedUrlCommand *)command {
    [[CAS settings] restartInterstitialInterval];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setAnalyticsCollectionEnabled:(CDVInvokedUrlCommand *)command {
    BOOL enable = [command argumentAtIndex:0];
    [[CAS settings] setAnalyticsCollectionWithEnabled:enable];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setTestDeviceIds:(CDVInvokedUrlCommand *)command {
    NSArray *testDevices = command.arguments;
    [[CAS settings] setTestDeviceWithIds:testDevices];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setUserGender:(CDVInvokedUrlCommand *)command {
    NSNumber *type = [command argumentAtIndex:0];
    [[CAS targetingOptions] setGender:(Gender)type];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setUserAge:(CDVInvokedUrlCommand *)command {
    NSNumber *age = [command argumentAtIndex:0];
    [[CAS targetingOptions] setAge:[age intValue]];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) getActiveMediationPattern:(CDVInvokedUrlCommand *)command {
    NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
    [data setObject:[CASNetwork getActiveNetworkPattern] forKey:@"isActiveMediationNetwork"];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) isActiveMediationNetwork:(CDVInvokedUrlCommand *)command {
    NSNumber *network = [command argumentAtIndex:0];
    NSInteger net = [network intValue];
    BOOL isActive = false;

    NSArray *values = [CASNetwork values];
    if (net > 0 && net < [values count]) {
        isActive = [CASNetwork isActiveNetwork:[values objectAtIndex:net]];
    }

    NSMutableDictionary *data = [[NSMutableDictionary alloc]init];
    [data setObject:[NSNumber numberWithBool:isActive] forKey:@"isActiveMediationNetwork"];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:data];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

- (void) setAllowInterstitialAdsWhenRewardedCostAreLower:(CDVInvokedUrlCommand *)command {
    BOOL allow = [command argumentAtIndex:0];
    [[CAS settings] setInterstitialAdsWhenVideoCostAreLowerWithAllow:allow];

    CDVPluginResult *result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsArray:command.arguments];
    [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
}

#pragma mark private methods

- (void)createBannerView {
    UIView *cordovaView = self.viewController.view;
    _bannerView = [[CASBannerView alloc] initWithAdSize: _bannerActiveSize manager: _mediationManager];
    _bannerView.translatesAutoresizingMaskIntoConstraints = NO;
    _bannerView.rootViewController = self.viewController;
    _bannerView.adDelegate = self.bannerCallback;
    [cordovaView addSubview:self.bannerView];
}

- (void)setBannerPosition:(NSInteger)positionId {
    bannerPositionId = positionId;
    if (!self.bannerView) {
        return;
    }
    UIView *cordovaView = self.bannerView.superview;
    if (cordovaView) {
        switch (positionId) {
            case 0:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.safeAreaLayoutGuide.topAnchor];
                    [self setHorizontalFor:_bannerView.centerXAnchor
                                        to:cordovaView.safeAreaLayoutGuide.centerXAnchor];
                } else {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.topAnchor];
                    [self setHorizontalFor:_bannerView.centerXAnchor
                                        to:cordovaView.centerXAnchor];
                }
                break;
            case 1:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.safeAreaLayoutGuide.topAnchor];
                    [self setHorizontalFor:_bannerView.leftAnchor
                                        to:cordovaView.safeAreaLayoutGuide.leftAnchor];
                } else {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.topAnchor];
                    [self setHorizontalFor:_bannerView.leftAnchor
                                        to:cordovaView.leftAnchor];
                }
                break;
            case 2:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.safeAreaLayoutGuide.topAnchor];
                    [self setHorizontalFor:_bannerView.rightAnchor
                                        to:cordovaView.safeAreaLayoutGuide.rightAnchor];
                } else {
                    [self setVerticalFor:_bannerView.topAnchor
                                      to:cordovaView.topAnchor];
                    [self setHorizontalFor:_bannerView.rightAnchor
                                        to:cordovaView.rightAnchor];
                }
                break;
            case 4:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.bottomAnchor to:cordovaView.safeAreaLayoutGuide.bottomAnchor];
                    [self setHorizontalFor:_bannerView.leftAnchor
                                        to:cordovaView.safeAreaLayoutGuide.leftAnchor];
                } else {
                    [self setVerticalFor:_bannerView.bottomAnchor
                                      to:cordovaView.bottomAnchor];
                    [self setHorizontalFor:_bannerView.leftAnchor
                                        to:cordovaView.leftAnchor];
                }
                break;
            case 5:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.bottomAnchor
                                      to:cordovaView.safeAreaLayoutGuide.bottomAnchor];
                    [self setHorizontalFor:_bannerView.rightAnchor
                                        to:cordovaView.safeAreaLayoutGuide.rightAnchor];
                } else {
                    [self setVerticalFor:_bannerView.bottomAnchor
                                      to:cordovaView.bottomAnchor];
                    [self setHorizontalFor:_bannerView.rightAnchor
                                        to:cordovaView.rightAnchor];
                }
                break;
            default:
                if (@available(iOS 11, *)) {
                    [self setVerticalFor:_bannerView.bottomAnchor
                                      to:cordovaView.safeAreaLayoutGuide.bottomAnchor];
                    [self setHorizontalFor:_bannerView.centerXAnchor
                                        to:cordovaView.safeAreaLayoutGuide.centerXAnchor];
                } else {
                    [self setVerticalFor:_bannerView.bottomAnchor
                                      to:cordovaView.bottomAnchor];
                    [self setHorizontalFor:_bannerView.centerXAnchor
                                        to:cordovaView.centerXAnchor];
                }
                break;
        }
        [cordovaView layoutIfNeeded ];
    }
}

- (void)setBannerSize:(NSInteger)sizeId {
    CASSize *targetSize;
    switch (sizeId) {
        case 1:
            targetSize = CASSize.banner;
            break;
        case 2:
            targetSize = [CASSize getAdaptiveBannerInContainer:self.viewController.view];
            break;
        case 3:
            targetSize = [CASSize getSmartBanner];
            break;
        case 4:
            targetSize = CASSize.leaderboard;
            break;
        case 5:
            targetSize = CASSize.mediumRectangle;
            break;
        default:
            NSLog(@"[CAS] Framework bridge call change banner size with unknown id: %zd", sizeId);
            return;
    }
    if (self.bannerActiveSize.height != targetSize.height || self.bannerActiveSize.width != targetSize.width) {
        self.bannerActiveSize = targetSize;
        [_bannerView setAdSize:targetSize];
    }
}

- (void)setHorizontalFor:(NSLayoutXAxisAnchor *)subview to:(NSLayoutXAxisAnchor *)container {
    if (self.horizontalConstraint) {
        self.horizontalConstraint.active = false;
    }
    self.horizontalConstraint = [subview constraintEqualToAnchor:container];
    self.horizontalConstraint.active = YES;
}

- (void)setVerticalFor:(NSLayoutYAxisAnchor *)subview to:(NSLayoutYAxisAnchor *)container {
    if (self.verticalConstraint) {
        self.verticalConstraint.active = false;
    }
    self.verticalConstraint = [subview constraintEqualToAnchor:container];
    self.verticalConstraint.active = YES;
}

- (void)onAdLoaded:(enum CASType)adType {
    NSString *result = [NSString stringWithFormat:@"%ld", (long)adType];
    result = [result stringByAppendingString:@"_Loaded"];

    [self fireEventTrigger:result];
}

- (void)onAdFailedToLoad:(enum CASType) adType withError:(NSString *)error {
    NSString *result = [NSString stringWithFormat:@"%ld", (long)adType];
    result = [result stringByAppendingString:@"_LoadFailed"];

    [self fireEventTrigger:result];
}

- (void)fireEventTrigger:(NSString *)event
{
    NSString *js = [NSString stringWithFormat:@"cordova.fireWindowEvent('%@')", event];
    [self.commandDelegate evalJs:js];
}
@end
